@@ -64,10 +64,6 @@ struct _PanelToplevelClass {
 	gboolean  (*toggle_hidden)    (PanelToplevel *toplevel);
 	gboolean  (*begin_move)       (PanelToplevel *toplevel);
 	gboolean  (*begin_resize)     (PanelToplevel *toplevel);
-
-	/* signals */
-	void      (*hiding)           (PanelToplevel *toplevel);
-	void      (*unhiding)         (PanelToplevel *toplevel);
 };
 
 GType                panel_toplevel_get_type               (void) G_GNUC_CONST;
@@ -93,14 +89,6 @@ void                 panel_toplevel_set_monitor            (PanelToplevel
 int                  panel_toplevel_get_monitor            (PanelToplevel       *toplevel);
 void                 panel_toplevel_set_auto_hide_size     (PanelToplevel       *toplevel,
 							    int                  autohide_size);
-void                 panel_toplevel_set_x                  (PanelToplevel       *toplevel,
-							    int                  x,
-							    int                  x_right,
-							    gboolean             x_centered);
-void                 panel_toplevel_set_y                  (PanelToplevel       *toplevel,
-							    int                  y,
-							    int                  y_bottom,
-							    gboolean             y_centered);
 
 gboolean             panel_toplevel_get_is_hidden          (PanelToplevel       *toplevel);
 
