@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2001 Sun Microsystems, Inc.
  * Copyright (c) 2010 Carlos Garcia Campos
- * Copyright (C) 2016-2020 Alberts Muktupāvels
+ * Copyright (C) 2016-2021 Alberts Muktupāvels
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -86,6 +86,8 @@ typedef struct
   guint               panel_icon_size;
   guint               menu_icon_size;
 
+  guint               icon_resize_id;
+
   GtkWidget          *about_dialog;
 } GpAppletPrivate;
 
@@ -140,13 +142,7 @@ update_enable_tooltips (GpApplet *applet)
   enable_tooltips = g_settings_get_boolean (priv->general_settings,
                                             "enable-tooltips");
 
-  if (priv->enable_tooltips == enable_tooltips)
-    return;
-
-  priv->enable_tooltips = enable_tooltips;
-
-  g_object_notify_by_pspec (G_OBJECT (applet),
-                            properties[PROP_ENABLE_TOOLTIPS]);
+  gp_applet_set_enable_tooltips (applet, enable_tooltips);
 }
 
 static void
@@ -159,13 +155,7 @@ update_prefer_symbolic_icons (GpApplet *applet)
   prefer_symbolic_icons = g_settings_get_boolean (priv->general_settings,
                                                   "prefer-symbolic-icons");
 
-  if (priv->prefer_symbolic_icons == prefer_symbolic_icons)
-    return;
-
-  priv->prefer_symbolic_icons = prefer_symbolic_icons;
-
-  g_object_notify_by_pspec (G_OBJECT (applet),
-                            properties[PROP_PREFER_SYMBOLIC_ICONS]);
+  gp_applet_set_prefer_symbolic_icons (applet, prefer_symbolic_icons);
 }
 
 static void
@@ -178,13 +168,7 @@ update_menu_icon_size (GpApplet *applet)
   menu_icon_size = g_settings_get_enum (priv->general_settings,
                                         "menu-icon-size");
 
-  if (priv->menu_icon_size == menu_icon_size)
-    return;
-
-  priv->menu_icon_size = menu_icon_size;
-
-  g_object_notify_by_pspec (G_OBJECT (applet),
-                            properties[PROP_MENU_ICON_SIZE]);
+  gp_applet_set_menu_icon_size (applet, menu_icon_size);
 }
 
 static void
@@ -232,13 +216,7 @@ update_panel_icon_size (GpApplet *applet)
       panel_icon_size = panel_max_icon_size;
     }
 
-  if (priv->panel_icon_size == panel_icon_size)
-    return;
-
-  priv->panel_icon_size = panel_icon_size;
-
-  g_object_notify_by_pspec (G_OBJECT (applet),
-                            properties[PROP_PANEL_ICON_SIZE]);
+  gp_applet_set_panel_icon_size (applet, panel_icon_size);
 }
 
 static void
@@ -259,6 +237,21 @@ general_settings_changed_cb (GSettings   *settings,
     update_panel_icon_size (applet);
 }
 
+static gboolean
+icon_resize_cb (gpointer user_data)
+{
+  GpApplet *self;
+  GpAppletPrivate *priv;
+
+  self = GP_APPLET (user_data);
+  priv = gp_applet_get_instance_private (self);
+
+  update_panel_icon_size (self);
+  priv->icon_resize_id = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
 static gboolean
 emit_size_hints_changed_cb (gpointer user_data)
 {
@@ -375,6 +368,12 @@ gp_applet_dispose (GObject *object)
       priv->size_hints_idle = 0;
     }
 
+  if (priv->icon_resize_id != 0)
+    {
+      g_source_remove (priv->icon_resize_id);
+      priv->icon_resize_id = 0;
+    }
+
   g_clear_pointer (&priv->initial_settings, g_variant_unref);
   g_clear_object (&priv->general_settings);
 
@@ -612,7 +611,7 @@ gp_applet_get_request_mode (GtkWidget *widget)
   applet = GP_APPLET (widget);
   priv = gp_applet_get_instance_private (applet);
 
-  if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (priv->orientation == GTK_ORIENTATION_VERTICAL)
     return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
 
   return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
@@ -622,13 +621,29 @@ static void
 gp_applet_size_allocate (GtkWidget     *widget,
                          GtkAllocation *allocation)
 {
-  GpApplet *applet;
+  GpApplet *self;
+  GpAppletPrivate *priv;
+  GtkAllocation old_allocation;
 
-  applet = GP_APPLET (widget);
+  self = GP_APPLET (widget);
+  priv = gp_applet_get_instance_private (self);
+
+  gtk_widget_get_allocation (widget, &old_allocation);
 
   GTK_WIDGET_CLASS (gp_applet_parent_class)->size_allocate (widget, allocation);
 
-  update_panel_icon_size (applet);
+  if ((priv->orientation == GTK_ORIENTATION_HORIZONTAL &&
+       old_allocation.height != allocation->height) ||
+      (priv->orientation == GTK_ORIENTATION_VERTICAL &&
+       old_allocation.width != allocation->width))
+    {
+      if (priv->icon_resize_id == 0)
+        {
+          priv->icon_resize_id = g_idle_add (icon_resize_cb, self);
+          g_source_set_name_by_id (priv->icon_resize_id,
+                                   "[libgnome-panel] icon_resize_cb");
+        }
+    }
 }
 
 static void
@@ -1417,6 +1432,23 @@ gp_applet_get_prefer_symbolic_icons (GpApplet *applet)
   return priv->prefer_symbolic_icons;
 }
 
+void
+gp_applet_set_prefer_symbolic_icons (GpApplet *self,
+                                     gboolean  prefer_symbolic_icons)
+{
+  GpAppletPrivate *priv;
+
+  priv = gp_applet_get_instance_private (self);
+
+  if (priv->prefer_symbolic_icons == prefer_symbolic_icons)
+    return;
+
+  priv->prefer_symbolic_icons = prefer_symbolic_icons;
+
+  g_object_notify_by_pspec (G_OBJECT (self),
+                            properties[PROP_PREFER_SYMBOLIC_ICONS]);
+}
+
 /**
  * gp_applet_get_panel_icon_size:
  * @applet: a #GpApplet
@@ -1436,6 +1468,23 @@ gp_applet_get_panel_icon_size (GpApplet *applet)
   return priv->panel_icon_size;
 }
 
+void
+gp_applet_set_panel_icon_size (GpApplet *self,
+                               guint     panel_icon_size)
+{
+  GpAppletPrivate *priv;
+
+  priv = gp_applet_get_instance_private (self);
+
+  if (priv->panel_icon_size == panel_icon_size)
+    return;
+
+  priv->panel_icon_size = panel_icon_size;
+
+  g_object_notify_by_pspec (G_OBJECT (self),
+                            properties[PROP_PANEL_ICON_SIZE]);
+}
+
 /**
  * gp_applet_get_menu_icon_size:
  * @applet: a #GpApplet
@@ -1455,6 +1504,23 @@ gp_applet_get_menu_icon_size (GpApplet *applet)
   return priv->menu_icon_size;
 }
 
+void
+gp_applet_set_menu_icon_size (GpApplet *self,
+                              guint     menu_icon_size)
+{
+  GpAppletPrivate *priv;
+
+  priv = gp_applet_get_instance_private (self);
+
+  if (priv->menu_icon_size == menu_icon_size)
+    return;
+
+  priv->menu_icon_size = menu_icon_size;
+
+  g_object_notify_by_pspec (G_OBJECT (self),
+                            properties[PROP_MENU_ICON_SIZE]);
+}
+
 /**
  * gp_applet_show_about:
  * @applet: a #GpApplet
@@ -1561,3 +1627,20 @@ gp_applet_popup_menu_at_widget (GpApplet  *applet,
                             widget_anchor, menu_anchor,
                             event);
 }
+
+void
+gp_applet_set_enable_tooltips (GpApplet *self,
+                               gboolean  enable_tooltips)
+{
+  GpAppletPrivate *priv;
+
+  priv = gp_applet_get_instance_private (self);
+
+  if (priv->enable_tooltips == enable_tooltips)
+    return;
+
+  priv->enable_tooltips = enable_tooltips;
+
+  g_object_notify_by_pspec (G_OBJECT (self),
+                            properties[PROP_ENABLE_TOOLTIPS]);
+}
