@@ -55,6 +55,7 @@
 
 #include "calendar-window.h"
 #include "clock-location.h"
+#include "clock-location-entry.h"
 #include "clock-location-tile.h"
 #include "clock-map.h"
 #include "clock-utils.h"
@@ -88,8 +89,7 @@ struct _ClockApplet
 	GtkWidget *prefs_location_edit_button;
 	GtkWidget *prefs_location_remove_button;
 
-	GWeatherLocationEntry *location_entry;
-        GWeatherTimezoneMenu *zone_combo;
+	ClockLocationEntry *location_entry;
 
 	GtkWidget *time_settings_button;
 	GAppInfo *datetime_appinfo;
@@ -446,7 +446,7 @@ create_cities_store (ClockApplet *cd)
 		gtk_list_store_set (cd->cities_store, &iter,
 				    COL_CITY_NAME, clock_location_get_name (loc),
 				    /* FIXME: translate the timezone */
-				    COL_CITY_TZ, clock_location_get_timezone (loc),
+				    COL_CITY_TZ, clock_location_get_timezone_identifier (loc),
                                     COL_CITY_LOC, loc,
 				    -1);
 
@@ -980,7 +980,7 @@ fill_clock_applet (ClockApplet *cd)
 
         cd->applet_settings = gp_applet_settings_new (applet, "org.gnome.gnome-panel.applet.clock");
         cd->clock_settings = g_settings_new ("org.gnome.desktop.interface");
-        cd->weather_settings = g_settings_new ("org.gnome.GWeather");
+        cd->weather_settings = g_settings_new ("org.gnome.GWeather4");
 
         g_signal_connect (cd->clock_settings, "changed::clock-format",
                           G_CALLBACK (format_changed), cd);
@@ -1076,8 +1076,7 @@ edit_clear (ClockApplet *cd)
         GtkWidget *lon_combo = _clock_get_widget (cd, "edit-location-longitude-combo");
 
         /* clear out the old data */
-        gweather_location_entry_set_location (cd->location_entry, NULL);
-        gweather_timezone_menu_set_tzid (cd->zone_combo, NULL);
+        clock_location_entry_set_location (cd->location_entry, NULL);
 
         gtk_entry_set_text (GTK_ENTRY (lat_entry), "");
         gtk_entry_set_text (GTK_ENTRY (lon_entry), "");
@@ -1107,17 +1106,10 @@ get_weather_station_location (GWeatherLocation *location)
          * is the nearest weather station.
          */
         if (gweather_location_get_level (location) == GWEATHER_LOCATION_DETACHED) {
-#ifdef HAVE_GWEATHER_40
                 station_loc = gweather_location_get_parent (location);
                 g_assert (station_loc != NULL);
-#else
-                station_loc = gweather_location_get_parent (location);
-                g_assert (station_loc != NULL);
-
-                station_loc = gweather_location_ref (station_loc);
-#endif
         } else {
-                station_loc = gweather_location_ref (location);
+                station_loc = g_object_ref (location);
         }
 
         while (gweather_location_get_level (station_loc) < GWEATHER_LOCATION_WEATHER_STATION) {
@@ -1125,17 +1117,10 @@ get_weather_station_location (GWeatherLocation *location)
 
                 tmp = station_loc;
 
-#ifdef HAVE_GWEATHER_40
                 station_loc = gweather_location_next_child (station_loc, NULL);
                 g_assert (station_loc != NULL);
-#else
-                station_loc = gweather_location_get_children (station_loc)[0];
-                g_assert (station_loc != NULL);
 
-                station_loc = gweather_location_ref (station_loc);
-#endif
-
-                gweather_location_unref (tmp);
+                g_object_unref (tmp);
         }
 
         return station_loc;
@@ -1154,7 +1139,7 @@ run_prefs_edit_save (GtkButton   *button,
         GtkWidget *lat_combo = _clock_get_widget (cd, "edit-location-latitude-combo");
         GtkWidget *lon_combo = _clock_get_widget (cd, "edit-location-longitude-combo");
 
-        const gchar *timezone, *weather_code;
+        const char *weather_code;
         gchar *city, *name;
 
         GWeatherLocation *gloc, *station_loc;
@@ -1166,29 +1151,23 @@ run_prefs_edit_save (GtkButton   *button,
                 g_object_unref (loc);
         }
 
-        timezone = gweather_timezone_menu_get_tzid (cd->zone_combo);
-        if (!timezone) {
-                edit_hide (NULL, cd);
-                return;
-        }
-
         city = NULL;
         weather_code = NULL;
         name = NULL;
 
-        gloc = gweather_location_entry_get_location (cd->location_entry);
+        gloc = clock_location_entry_get_location (cd->location_entry);
         if (!gloc) {
                 edit_hide (NULL, cd);
                 return;
         }
 
         station_loc = get_weather_station_location (gloc);
-        gweather_location_unref (gloc);
+        g_object_unref (gloc);
 
         weather_code = gweather_location_get_code (station_loc);
-        gweather_location_unref (station_loc);
+        g_object_unref (station_loc);
 
-        if (gweather_location_entry_has_custom_text (cd->location_entry)) {
+        if (clock_location_entry_has_custom_text (cd->location_entry)) {
                 name = gtk_editable_get_chars (GTK_EDITABLE (cd->location_entry), 0, -1);
         }
 
@@ -1268,31 +1247,17 @@ update_coords (ClockApplet *cd,
 	update_coords_helper (lon, lon_entry, lon_combo);
 }
 
-static void
-fill_timezone_combo_from_location (ClockApplet   *cd,
-                                   ClockLocation *loc)
-{
-        if (loc != NULL) {
-                gweather_timezone_menu_set_tzid (cd->zone_combo, 
-                                                 clock_location_get_timezone (loc));
-        } else {
-                gweather_timezone_menu_set_tzid (cd->zone_combo, NULL);
-        }
-}
-
 static void
 location_update_ok_sensitivity (ClockApplet *cd)
 {
 	GtkWidget *ok_button;
-        const gchar *timezone;
         gchar *name;
 
         ok_button = _clock_get_widget (cd, "edit-location-ok-button");
 
-        timezone = gweather_timezone_menu_get_tzid (cd->zone_combo);
         name = gtk_editable_get_chars (GTK_EDITABLE (cd->location_entry), 0, -1);
 
-        if (timezone && name && name[0] != '\0') {
+        if (name && name[0] != '\0') {
                 gtk_widget_set_sensitive (ok_button, TRUE);
         } else {
                 gtk_widget_set_sensitive (ok_button, FALSE);
@@ -1306,27 +1271,20 @@ location_changed (GObject     *object,
                   GParamSpec  *param,
                   ClockApplet *cd)
 {
-        GWeatherLocationEntry *entry = GWEATHER_LOCATION_ENTRY (object);
+        ClockLocationEntry *entry = CLOCK_LOCATION_ENTRY (object);
         GWeatherLocation *gloc;
-        GWeatherTimezone *zone;
         gboolean latlon_valid;
         double latitude = 0.0, longitude = 0.0;
 
-        gloc = gweather_location_entry_get_location (entry);
+        gloc = clock_location_entry_get_location (entry);
 
 	latlon_valid = gloc && gweather_location_has_coords (gloc);
         if (latlon_valid)
                 gweather_location_get_coords (gloc, &latitude, &longitude);
         update_coords (cd, latlon_valid, latitude, longitude);
 
-        zone = gloc ? gweather_location_get_timezone (gloc) : NULL;
-        if (zone)
-                gweather_timezone_menu_set_tzid (cd->zone_combo, gweather_timezone_get_tzid (zone));
-        else
-                gweather_timezone_menu_set_tzid (cd->zone_combo, NULL);
-
         if (gloc)
-                gweather_location_unref (gloc);
+                g_object_unref (gloc);
 }
 
 static void
@@ -1336,14 +1294,6 @@ location_name_changed (GObject     *object,
     location_update_ok_sensitivity (cd);
 }
 
-static void
-location_timezone_changed (GObject     *object,
-                           GParamSpec  *param,
-                           ClockApplet *cd)
-{
-    location_update_ok_sensitivity (cd);
-}
-
 static gboolean
 edit_delete (GtkWidget   *unused,
              GdkEvent    *event,
@@ -1424,8 +1374,6 @@ run_prefs_locations_add (GtkButton   *button,
 {
         GtkWidget *edit_window = _clock_get_widget (cd, "edit-location-window");
 
-        fill_timezone_combo_from_location (cd, NULL);
-
         g_object_set_data (G_OBJECT (edit_window), "clock-location", NULL);
         gtk_window_set_title (GTK_WINDOW (edit_window), _("Choose Location"));
         gtk_window_set_transient_for (GTK_WINDOW (edit_window), GTK_WINDOW (cd->prefs_window));
@@ -1467,9 +1415,9 @@ edit_tree_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
 
         gtk_tree_model_get (model, iter, COL_CITY_LOC, &loc, -1);
 
-        gweather_location_entry_set_city (cd->location_entry,
-                                          clock_location_get_city (loc),
-                                          clock_location_get_weather_code (loc));
+        clock_location_entry_set_city (cd->location_entry,
+                                       clock_location_get_city (loc),
+                                       clock_location_get_weather_code (loc));
 	name = clock_location_get_name (loc);
         if (name && name[0]) {
                 gtk_entry_set_text (GTK_ENTRY (cd->location_entry), name);
@@ -1477,8 +1425,6 @@ edit_tree_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
 
         clock_location_get_coords (loc, &lat, &lon);
 
-        fill_timezone_combo_from_location (cd, loc);
-
         tmp = g_strdup_printf ("%f", fabs (lat));
         gtk_entry_set_text (GTK_ENTRY (lat_entry), tmp);
         g_free (tmp);
@@ -1672,9 +1618,7 @@ ensure_prefs_window_is_created (ClockApplet *cd)
         GtkWidget *edit_cancel_button;
         GtkWidget *edit_ok_button;
         GtkWidget *location_box;
-        GtkWidget *zone_box;
         GtkWidget *location_name_label;
-        GtkWidget *timezone_label;
         GtkTreeSelection *selection;
 
         if (cd->prefs_window)
@@ -1689,8 +1633,6 @@ ensure_prefs_window_is_created (ClockApplet *cd)
         clock_options = _clock_get_widget (cd, "clock-options");
         cd->prefs_locations = GTK_TREE_VIEW (_clock_get_widget (cd, "cities_list"));
         location_name_label = _clock_get_widget (cd, "location-name-label");
-        timezone_label = _clock_get_widget (cd, "timezone-label");
-
 
 	if (!clock_locale_supports_am_pm ())
 		gtk_widget_hide (clock_options);
@@ -1736,7 +1678,7 @@ ensure_prefs_window_is_created (ClockApplet *cd)
         edit_ok_button = _clock_get_widget (cd, "edit-location-ok-button");
 
         location_box = _clock_get_widget (cd, "edit-location-name-box");
-        cd->location_entry = GWEATHER_LOCATION_ENTRY (gweather_location_entry_new (cd->world));
+        cd->location_entry = CLOCK_LOCATION_ENTRY (clock_location_entry_new (cd->world));
         gtk_widget_show (GTK_WIDGET (cd->location_entry));
         gtk_container_add (GTK_CONTAINER (location_box), GTK_WIDGET (cd->location_entry));
         gtk_label_set_mnemonic_widget (GTK_LABEL (location_name_label),
@@ -1747,16 +1689,6 @@ ensure_prefs_window_is_created (ClockApplet *cd)
         g_signal_connect (G_OBJECT (cd->location_entry), "changed",
                           G_CALLBACK (location_name_changed), cd);
 
-        zone_box = _clock_get_widget (cd, "edit-location-timezone-box");
-        cd->zone_combo = GWEATHER_TIMEZONE_MENU (gweather_timezone_menu_new (cd->world));
-        gtk_widget_show (GTK_WIDGET (cd->zone_combo));
-        gtk_container_add (GTK_CONTAINER (zone_box), GTK_WIDGET (cd->zone_combo));
-        gtk_label_set_mnemonic_widget (GTK_LABEL (timezone_label),
-                                       GTK_WIDGET (cd->zone_combo));
-
-        g_signal_connect (G_OBJECT (cd->zone_combo), "notify::tzid",
-                          G_CALLBACK (location_timezone_changed), cd);
-
         g_signal_connect (G_OBJECT (edit_cancel_button), "clicked",
                           G_CALLBACK (edit_hide), cd);
 
@@ -1816,9 +1748,7 @@ clock_applet_dispose (GObject *object)
 
         g_clear_object (&applet->datetime_appinfo);
 
-#ifdef HAVE_GWEATHER_40
-        g_clear_pointer (&applet->world, gweather_location_unref);
-#endif
+        g_clear_object (&applet->world);
 
         free_locations (applet);
 
